call omake attiny841at184
call omake attiny841at147
call omake attiny841at110
call omake attiny841at921
call omake attiny841at737
call omake attiny841at20
call omake attiny841at16
call omake attiny841at12
call omake attiny841at8
call omake attiny841at8_int
call omake attiny841at1
call omake attiny841at1_int

call omake attiny841at184ser1
call omake attiny841at147ser1
call omake attiny841at110ser1
call omake attiny841at921ser1
call omake attiny841at737ser1
call omake attiny841at20ser1
call omake attiny841at16ser1
call omake attiny841at12ser1
call omake attiny841at8ser1
call omake attiny841at1ser1
call omake attiny841at8_intser1
call omake attiny841at1_intser1

call omake attiny441at184
call omake attiny441at147
call omake attiny441at110
call omake attiny441at921
call omake attiny441at737
call omake attiny441at20
call omake attiny441at16
call omake attiny441at12
call omake attiny441at8
call omake attiny441at8_int
call omake attiny441at1
call omake attiny441at1_int

call omake attiny441at184ser1
call omake attiny441at147ser1
call omake attiny441at110ser1
call omake attiny441at921ser1
call omake attiny441at737ser1
call omake attiny441at20ser1
call omake attiny441at16ser1
call omake attiny441at12ser1
call omake attiny441at8ser1
call omake attiny441at1ser1
call omake attiny441at8_intser1
call omake attiny441at1_intser1

call omake attiny87at184
call omake attiny87at147
call omake attiny87at110
call omake attiny87at921
call omake attiny87at737
call omake attiny87at20
call omake attiny87at16
call omake attiny87at12
call omake attiny87at8
call omake attiny87at1

call omake attiny167at184
call omake attiny167at147
call omake attiny167at110
call omake attiny167at921
call omake attiny167at737
call omake attiny167at20
call omake attiny167at16
call omake attiny167at12
call omake attiny167at8
call omake attiny167at1

call omake attiny1634at147
call omake attiny1634at110
call omake attiny1634at921
call omake attiny1634at737
call omake attiny1634at20
call omake attiny1634at16
call omake attiny1634at12
call omake attiny1634at8
call omake attiny1634at8_int
call omake attiny1634at1
call omake attiny1634at1_int

call omake attiny1634at147ser1
call omake attiny1634at110ser1
call omake attiny1634at921ser1
call omake attiny1634at737ser1
call omake attiny1634at20ser1
call omake attiny1634at16ser1
call omake attiny1634at12ser1
call omake attiny1634at8ser1
call omake attiny1634at8_intser1
call omake attiny1634at1ser1
call omake attiny1634at1_intser1


call omake attiny828at8
call omake attiny828at12
call omake attiny828at16
call omake attiny828at20
call omake attiny828at8_int
call omake attiny828at1
call omake attiny828at1_int

call omake attiny88at20
call omake attiny88at16
call omake attiny88at12
call omake attiny88at8
call omake attiny88at4
call omake attiny88at1

call omake attiny48at20
call omake attiny48at16
call omake attiny48at12
call omake attiny48at8
call omake attiny48at4
call omake attiny48at1

call omake attiny85at184
call omake attiny85at147
call omake attiny85at110
call omake attiny85at921
call omake attiny85at737
call omake attiny85at20
call omake attiny85at16
call omake attiny85at12
call omake attiny85at8
call omake attiny85at1

call omake attiny45at184
call omake attiny45at147
call omake attiny45at110
call omake attiny45at921
call omake attiny45at737
call omake attiny45at20
call omake attiny45at16
call omake attiny45at12
call omake attiny45at8
call omake attiny45at1

call omake attiny84at184
call omake attiny84at147
call omake attiny84at110
call omake attiny84at921
call omake attiny84at737
call omake attiny84at20
call omake attiny84at16
call omake attiny84at12
call omake attiny84at8
call omake attiny84at1

call omake attiny44at184
call omake attiny44at147
call omake attiny44at110
call omake attiny44at921
call omake attiny44at737
call omake attiny44at20
call omake attiny44at16
call omake attiny44at12
call omake attiny44at8
call omake attiny44at1

call omake attiny861at184
call omake attiny861at147
call omake attiny861at110
call omake attiny861at921
call omake attiny861at737
call omake attiny861at20
call omake attiny861at16
call omake attiny861at12
call omake attiny861at8
call omake attiny861at1

call omake attiny461at184
call omake attiny461at147
call omake attiny461at110
call omake attiny461at921
call omake attiny461at737
call omake attiny461at20
call omake attiny461at16
call omake attiny461at12
call omake attiny461at8
call omake attiny461at1
